BuidDot ... l'intégration continue
==================================
![category](développement)
![tag](devops)
![tag](docker)

Il est indispensable quand on developpe d'avoir une stack technique.

Cette stack doit permettre une intégration et un déploiement continue.

Pour cela il est nécessaire d'avoir un outil qui puisse en continue réaliser après un commit

- des tests
- du packaging
- du déploiement

Un des principaux outils pour ce genre de chose est *Jenkins*. Il est performant, bien maintenue
mais possède pour moi de nombreux défauts: java, configuratio difficile, ..

Un autre outil est *buildbot* ... et il a de nombreux avantages

- écrit en python
- distribué
- maintenue
- configuration par programmation
- ...

Son principe de fonctionnement repose sur

- un ou plusieurs master qui distribue des tâches (factory)
- sur un ou plusieurs workers
- les taches sont émise via un planificateur (scheduler) lui même relié
- un serveur de source

Dans notre exemple, nous allons mettre en place via docker

- un serveur git
- un master buildbot
- un worker buildot avec python 2.7
- un worker buildot avec python 3.5

chaque worker aura un factory réalisant les taches

- récupération des sources
- vérification version python
- lancement du programe *test.py*

.. aafig::
    :textual:

                    /----------\
                    | gitweb   |
                    \-----+----/
                          |
      /----------\  /-----+----\ 
      |    db    |  | buildbot | 
      \-----+----/  \-----+----/ 
            |             |           
            +-------------+
                          |
            +-------------+
            |             |
      /-----+------\  /---+-------\
      | worker35   |  | worker27  |
      \------------/  \-----------/


On pourra accéder

- à buildbot via http://localhost:8010/
- à gitweb via http://localhost/

Mise en oeuvre
--------------

la mise en oeuvre emploie bien sur docker et docker-compose

Notre projet se présente ainsi

![201804071741_01.png](./upload/201804071741_01.png)

Notre fichier docker-compose.yml est le suivant

```
version: '2'
services:
buildbot:
    image: buildbot/buildbot-master:master
    environment:
    - BUILDBOT_CONFIG_DIR=config
    - BUILDBOT_CONFIG_URL=http://gituser:gitpassword@gitweb/?p=myproject.git;a=blob_plain;f=master.cfg
    - BUILDBOT_WORKER_PORT=9989
    - BUILDBOT_WEB_URL=http://localhost:8010/
    - BUILDBOT_WEB_PORT=8010
    - POSTGRES_PASSWORD=change_me
    - POSTGRES_USER=buildbot
    - POSTGRES_DB=buildbot
    - BUILDBOT_DB_URL=postgresql+psycopg2://{POSTGRES_USER}:{POSTGRES_PASSWORD}@db/{POSTGRES_DB}
    depends_on:
    - db
    - gitweb
    ports:
    - "8010:8010"
    - "9989:9989"
db:
    environment:
    - POSTGRES_PASSWORD=change_me
    - POSTGRES_USER=buildbot
    - POSTGRES_DB=buildbot
    image: "postgres:9.4"
    ports:
    - "5432:5432"

worker:
    image: "buildbot/buildbot-worker:master"
    environment:
    - BUILDMASTER=buildbot
    - BUILDMASTER_PORT=9989
    - WORKERNAME=worker27
    - WORKERPASS=password27
    - WORKER_ENVIRONMENT_BLACKLIST=DOCKER_BUILDBOT* BUILDBOT_ENV_* BUILDBOT_1* WORKER_ENVIRONMENT_BLACKLIST
    depends_on:
    - buildbot

workerpy35:
    build: 
    context: workerpy35
    environment:
    - BUILDMASTER=buildbot
    - BUILDMASTER_PORT=9989
    - WORKERNAME=worker35
    - WORKERPASS=password35
    - WORKER_ENVIRONMENT_BLACKLIST=DOCKER_BUILDBOT* BUILDBOT_ENV_* BUILDBOT_1* WORKER_ENVIRONMENT_BLACKLIST
    depends_on:
    - buildbot

gitweb:
    build: 
    context: gitweb
    ports:
        - "80:80"
    environment:
        - GITPROJECT=myproject
        - GITUSER=gituser
        - GITPASSWORD=gitpassword
```

```note
on voit que les worker sont protégés par un login/password qu'On
retrouvera dans le master.cfg
```

Notre fichier gitweb/Dockerfile

```
FROM fraoustin/gitweb
COPY ./00_init.sh /usr/share/gitweb/docker-entrypoint.pre/00_init.sh
RUN chmod +x -R /usr/share/gitweb/docker-entrypoint.pre
COPY ./master.cfg /master.cfg
```

Notre fichier gitweb/00_init.sh qui va permettre de créer le répository puis d'ajouter
le fichier master.cfg. Cela est important car au démarrage, le master de buildbot va
télécharger ce fichier pour lancer sa configuration

Notre fichier gitweb/00_init.sh

```
#!/bin/bash
if [ ! -z "$GITPROJECT" ]; then
    REPOS='/var/lib/git/'$GITPROJECT'.git'
    if [ ! -d $REPOS ]; then
        addrepos $GITPROJECT
        git config --global user.email $GITUSER"@gitweb.com"
        cd /tmp
        git clone $REPOS
        cd $GITPROJECT
        cp /master.cfg .
        git add *
        git commit -m "add master.cfg"
        git push
    fi
    addauth $GITUSER $GITPASSWORD
fi 
```

Notre fichier gitweb/master.cfg

```python
# -*- python -*-
# ex: set filetype=python:

import os

from buildbot.plugins import *

c = BuildmasterConfig = {}

####### WORKERS

c['workers'] = [
        worker.Worker("worker27", 'password27'), 
        worker.Worker("worker35", 'password35')
    ]

if 'BUILDBOT_MQ_URL' in os.environ:
    c['mq'] = {
        'type' : 'wamp',
        'router_url': os.environ['BUILDBOT_MQ_URL'],
        'realm': os.environ.get('BUILDBOT_MQ_REALM', 'buildbot').decode('utf-8'),
        'debug' : 'BUILDBOT_MQ_DEBUG' in os.environ,
        'debug_websockets' : 'BUILDBOT_MQ_DEBUG' in os.environ,
        'debug_lowlevel' : 'BUILDBOT_MQ_DEBUG' in os.environ,
    }

c['protocols'] = {'pb': {'port': os.environ.get("BUILDBOT_WORKER_PORT", 9989)}}

####### CHANGESOURCES

c['change_source'] = []
c['change_source'].append(changes.GitPoller(
        'http://gituser:gitpassword@gitweb/myproject.git',
        workdir='gitpoller-workdir', branch='master',
        pollinterval=10)) 
        # check all 10s the git server
        # check on ly branch master

####### SCHEDULERS

c['schedulers'] = []
c['schedulers'].append(schedulers.SingleBranchScheduler(
                            name="all",
                            change_filter=util.ChangeFilter(branch='master'),
                            treeStableTimer=None,
                            builderNames=["builder27","builder35"]))
c['schedulers'].append(schedulers.ForceScheduler(
                            name="force",
                            builderNames=["builder27","builder35"]))

####### BUILDERS

factory27 = util.BuildFactory()
factory27.addStep(steps.Git(repourl='http://gituser:gitpassword@gitweb/myproject.git', mode='incremental'))
factory27.addStep(steps.ShellCommand(name="check python", command=["python", "--version"]))
factory27.addStep(steps.ShellCommand(name="run test", command=["python", "test.py"]))

c['builders'] = []
c['builders'].append(
    util.BuilderConfig(name="builder27",
    workernames=["worker27"],
    factory=factory27))


factory35 = util.BuildFactory()
factory35.addStep(steps.Git(repourl='http://gituser:gitpassword@gitweb/myproject.git', mode='incremental'))
factory35.addStep(steps.ShellCommand(name="check python", command=["python3", "--version"]))
factory35.addStep(steps.ShellCommand(name="run test", command=["python3", "test.py"]))

c['builders'].append(
    util.BuilderConfig(name="builder35",
    workernames=["worker35"],
    factory=factory35))

####### STATUS TARGETS

c['status'] = []

####### PROJECT IDENTITY

c['title'] = "My Project"
c['titleURL'] = "http://localhost/"

c['buildbotURL'] = os.environ.get("BUILDBOT_WEB_URL", "http://localhost:8010/")

c['www'] = dict(port=os.environ.get("BUILDBOT_WEB_PORT", 8010),
                plugins=dict(waterfall_view={}, console_view={}))

####### DB URL

c['db'] = {
    'db_url' : os.environ.get("BUILDBOT_DB_URL", "sqlite://").format(**os.environ),
}
```

On retrouve bien, nos workers, nos factorys, nos builders.

Notre fichier postgres/db.env contient le nécessaire pour initier la base postgresql de buildbot

```
# database parameters are shared between containers
POSTGRES_PASSWORD=change_me
POSTGRES_USER=buildbot
POSTGRES_DB=buildbot
# in master.cfg, this variable is str.format()ed with the environment variables
BUILDBOT_DB_URL=postgresql+psycopg2://{POSTGRES_USER}:{POSTGRES_PASSWORD}@db/{POSTGRES_DB}
```

Notre fichier workerpy35/Dockerfile contient le nécessaire pour ajouter python35 dans notre worker

```
FROM buildbot/buildbot-worker:master

USER root

RUN apt-get update && \
    apt-get -y upgrade && \
    apt-get -y install -q \
        python3-dev \
        python3-pip \
    && rm -rf /var/lib/apt/lists/* 
```

Pour lancer notre stack rien de plus facile

```bash
docker-compose up -d
```

Usage
-----

Il suffit de récupérer notre projet, de lui ajouter un fichier test.py et de voir ce qui se passe

```
git clone http://gituser:gitpassword@127.0.0.1/myproject.git
cd myproject
echo "print 'hello'" > test.py
git add * && git commit -m "first test" && git push
```

Et une fois sur buildbot on constate

- que le builder27 est ok
- que le builder35 est ko

remedions à cela

```
echo "print('hello')" > test.py
git add * && git commit -m "correction" && git push 
```

Et une fois sur buildbot on constate

- que le builder27 est ok
- que le builder35 est ok

ous pouvez retrouver un projet rendant l'ensemble complètement automatique [ici](images/201804071741.zip)` 

```warning
il faut noter que dans notre cas avec gitweb on ne peut utiliser de hook. 
Notre serveur buildbot vérifie donc içi toutes les 10 secondes si il y a 
eu des changements sur la branche master.

On peut bien sur changer ce comportement et pour cela allez voir la doc officiel